@@ -1,5 +1,5 @@
 # Text To Gcode
-# Copyright (C) 2021  Ryan Zmuda
+# Copyright (C) 2022  Ryan Zmuda
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,18 +15,34 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-import time
+import sys
 from os import remove, rename
 from math import cos, sin, radians
 
 
 class ttg:
     def __init__(self, text, size, rotation, method, feedRate):
+        # validate input types
+        if type(text) != str:
+            raise Exception("TTG ERROR - passed text needs to be a string")
+
+        if type(size) != int:
+            raise Exception("TTG ERROR - passed size needs to be a int")
+
+        if type(rotation) != int:
+            raise Exception("TTG ERROR - passed size rotation to be a int")
+
+        if type(method) != str:
+            raise Exception("TTG ERROR - passed return method needs to be a string")
+
+        if type(feedRate) != int:
+            raise Exception("TTG ERROR - passed feed rate needs to be a int")
+
         # set basic passed args
         self.text = text
         self.size = size
         self.rotation = radians(rotation)
-        self.method = method
+        self.method = method.lower() # lowercase this for matching
         self.feedRate = str(feedRate)
 
         # set global class vars
@@ -107,7 +123,7 @@ class ttg:
                 gFile = open("output.txt", "a")  # create temp file
 
                 # stamp temp file
-                gFile.write(";generated by ttglib: Ryan Zmuda 2021\n")
+                gFile.write(";generated by ttglib: Ryan Zmuda 2022\n")
 
             # Add neccessary header commands
             # EXPLANATION:
@@ -204,6 +220,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 0),
@@ -222,7 +239,6 @@ class ttg:
             (0 + xOff, 4),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -243,6 +259,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 0),
@@ -286,6 +303,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "off",
             "fast",
             (5 + xOff, 1),
@@ -300,7 +318,6 @@ class ttg:
             (5 + xOff, 8),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -321,6 +338,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 9),
@@ -331,6 +349,7 @@ class ttg:
             (4 + xOff, 1),
             (3 + xOff, 0),
             (0 + xOff, 0),
+            (0 + xOff, 9),
             "off",
             "fast",
         ]
@@ -353,6 +372,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 9),
@@ -369,6 +389,7 @@ class ttg:
             "on",
             "slow",
             (0 + xOff, 0),
+            (0 + xOff, 9),
             "off",
             "fast",
         ]
@@ -391,6 +412,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 9),
@@ -404,6 +426,11 @@ class ttg:
             "off",
             "fast",
             (0 + xOff, 0),
+            "on",
+            "slow",
+            (0 + xOff, 9),
+            "off",
+            "fast",
         ]
 
         ttg.appendPoints(self, points)
@@ -440,7 +467,6 @@ class ttg:
             (4 + xOff, 4),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -461,6 +487,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 9),
@@ -475,10 +502,9 @@ class ttg:
             (0 + xOff, 5),
             "on",
             "slow",
-            (0 + xOff, 5),
+            (5 + xOff, 5),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -499,6 +525,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (4 + xOff, 0),
@@ -516,7 +543,6 @@ class ttg:
             (2 + xOff, 0),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -537,6 +563,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (1 + xOff, 0),
@@ -551,7 +578,6 @@ class ttg:
             (4 + xOff, 9),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -572,6 +598,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 9),
@@ -585,6 +612,7 @@ class ttg:
             (3 + xOff, 5),
             (2 + xOff, 4),
             (1 + xOff, 4),
+            (0 + xOff, 4),
             (2 + xOff, 4),
             (3 + xOff, 3),
             (4 + xOff, 2),
@@ -592,7 +620,6 @@ class ttg:
             (5 + xOff, 0),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -620,7 +647,6 @@ class ttg:
             (5 + xOff, 0),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -641,6 +667,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 9),
@@ -655,7 +682,6 @@ class ttg:
             (6 + xOff, 0),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -676,6 +702,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 9),
@@ -683,7 +710,6 @@ class ttg:
             (5 + xOff, 9),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -714,9 +740,9 @@ class ttg:
             (5 + xOff, 1),
             (4 + xOff, 0),
             (1 + xOff, 0),
+            (0 + xOff, 1),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -737,6 +763,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 8),
@@ -748,7 +775,6 @@ class ttg:
             (0 + xOff, 4),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -781,13 +807,15 @@ class ttg:
             (5 + xOff, 0),
             "off",
             "fast",
-            (3 + xOff, 0),
+            (4 + xOff, 1),
             "on",
             "slow",
+            (4 + xOff, 1),
+            (3 + xOff, 0),
             (1 + xOff, 0),
+            (0 + xOff, 1),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -808,6 +836,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (0 + xOff, 8),
@@ -829,7 +858,6 @@ class ttg:
             (5 + xOff, 0),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -850,6 +878,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (4 + xOff, 0),
@@ -863,7 +892,6 @@ class ttg:
             (5 + xOff, 9),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -896,7 +924,6 @@ class ttg:
             (4 + xOff, 9),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -927,7 +954,6 @@ class ttg:
             (5 + xOff, 9),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -955,7 +981,6 @@ class ttg:
             (4 + xOff, 9),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -989,7 +1014,6 @@ class ttg:
             (6 + xOff, 9),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1011,6 +1035,7 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (4 + xOff, 9),
@@ -1022,7 +1047,6 @@ class ttg:
             (4 + xOff, 0),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1058,7 +1082,6 @@ class ttg:
             (2 + xOff, 4),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1088,7 +1111,6 @@ class ttg:
             (5 + xOff, 0),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1122,7 +1144,6 @@ class ttg:
             (2 + xOff, 9),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1152,7 +1173,6 @@ class ttg:
             (0 + xOff, 8),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1193,7 +1213,6 @@ class ttg:
             (0 + xOff, 8),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1227,7 +1246,6 @@ class ttg:
             (4 + xOff, 0),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1294,7 +1312,6 @@ class ttg:
             (0 + xOff, 4),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1315,13 +1332,13 @@ class ttg:
         xOff = self.currentXOffset
 
         points = [
+            (0 + xOff, 0),
             "on",
             "slow",
             (4 + xOff, 9),
             (0 + xOff, 9),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1364,7 +1381,6 @@ class ttg:
             (3 + xOff, 0),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1396,7 +1412,6 @@ class ttg:
             (4 + xOff, 4),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1430,7 +1445,6 @@ class ttg:
             (0 + xOff, 1),
             "off",
             "fast",
-            (0 + xOff, 0),
         ]
 
         ttg.appendPoints(self, points)
@@ -1613,9 +1627,80 @@ class ttg:
             return ttg.collectCharacters(self)
 
     def toGcode(self, onPower, offPower, fast, slow):
+        # validate inputs
+        if type(onPower) != str:
+            raise Exception("TTG ERROR - passed on command needs to be a string")
+
+        if type(offPower) != str:
+            raise Exception("TTG ERROR - passed off command needs to be a string")
+
+        if type(fast) != str:
+            raise Exception("TTG ERROR - passed fast command needs to be a string")
+
+        if type(slow) != str:
+            raise Exception("TTG ERROR - passed slow command needs to be a string")
+
         self.onPowerCmd = onPower
         self.offPowerCmd = offPower
         self.fastCmd = fast
         self.slowCmd = slow
 
-        return ttg.encode(self)
\ No newline at end of file
+        return ttg.encode(self)
+
+
+if __name__ == "__main__":
+    # if the file is ran from the command line we need to get the args
+    args = sys.argv
+    args.pop(0)  # useless arg
+
+    # argument order:
+    # 0. text surrounded by ""
+    # 1. integer for scaling size of text
+    # 2. integer of the degrees of rotation of the text
+    # 3. string representing the mode of return (return, file or visualize)
+    # 4. integer representing feedrate
+    # 5. string for on command surrounded by ""
+    # 6. string for off command surrounded by ""
+    # 7. string for fast command surrounded by ""
+    # 8. string for slow command surrounded by ""
+
+    # ensure that we are running from the command line with all arguments
+    if args:
+        # if we have the correct amound of arguments supplied
+        if len(args) == 9:
+            try:
+                args[0] = str(args[0])
+                args[1] = int(args[1])
+                args[2] = int(args[2])
+                args[3] = str(args[3])
+                args[4] = int(args[4])
+                args[5] = str(args[5])
+                args[6] = str(args[6])
+                args[7] = str(args[7])
+                args[8] = str(args[8])
+
+            except:
+                print(
+                    "TTGLIB ERROR: something has gone wrong validating command line arguments"
+                )
+                exit()
+
+            # call the library to act on our parameters supplied in the arguments
+            print(
+                ttg(args[0], args[1], args[2], args[3], args[4]).toGcode(
+                    args[5], args[6], args[7], args[8]
+                )
+            )
+
+        else:
+            print(
+                "TTGLIB ERROR: you have supplied "
+                + str(len(args))
+                + " arguments when 9 are required to run"
+            )
+            exit()
+    else:
+        print(
+            "TTGLIB ERROR: This program is not intended to be ran directly. Please refer to the readme :)"
+        )
+        exit()
